/// <summary> </summary>
using System;
using org.solol.mmseg;
using AlgorithmException = org.solol.mmseg.core.AlgorithmException;
using Config = org.solol.mmseg.core.Config;
using ConfigurationException = org.solol.mmseg.core.ConfigurationException;
using DictionaryFactory = org.solol.mmseg.core.DictionaryFactory;
using IAlgorithm = org.solol.mmseg.core.IAlgorithm;
using IAmbiguityHandler = org.solol.mmseg.core.IAmbiguityHandler;
using IChunk = org.solol.mmseg.core.IChunk;
using IDictionary = org.solol.mmseg.core.IDictionary;
using IRule = org.solol.mmseg.core.IRule;
using IWord = org.solol.mmseg.core.IWord;
namespace org.solol.mmseg
{
	
	/// <author>  solo L
	/// 
	/// </author>
	public abstract class AbstractAlgorithm : IAlgorithm
	{
		virtual public IAmbiguityHandler AmbiguityHandler
		{
			set
			{
				this.ambiguityHandler = value;
			}
			
			/*	private boolean isCJK(char c) {
			return Character.UnicodeBlock.of(c) == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS
			|| Character.UnicodeBlock.of(c) == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION;
			}*/
			
		}
		
		protected internal DictionaryFactory dictionaryFactory;
		
		protected internal IRule mmRule = new MMRule();
		
		protected internal IRule lawlRule = new LAWLRule();
		
		protected internal IRule svwlRule = new SVWLRule();
		
		protected internal IRule lsdmfocwRule = new LSDMFOCWRule();
		
		protected internal int index = 0;
		
		protected internal IAmbiguityHandler ambiguityHandler = new DefaultAmbiguityHandler();
		
		public AbstractAlgorithm()
		{
			try
			{
				dictionaryFactory = DictionaryFactory.Factory;
			}
			catch (ConfigurationException e)
			{
				throw new AlgorithmException(e);
			}
		}
		
		public AbstractAlgorithm(DictionaryFactory dictionaryFactory)
		{
			this.dictionaryFactory = dictionaryFactory;
		}
		
		public virtual IWord next(char[] chars)
		{
			
			if (index >= chars.Length)
			{
				index = 0;
				return null;
			}
			
			char current = chars[index];
			
			if (isBasicLatin(current))
			{
				IWord word = getBasicLatinWord(chars, index);
				index += word.Length;
				return word;
			}
			else
			{
				IChunk[] chunks = createChunks(chars, index);
				
				IWord word = getCJKWord(chunks);
				
				index += word.Length;
				
				return word;
			}
		}
		
		protected internal virtual IWord getBasicLatinWord(char[] chars, int index)
		{
			System.Text.StringBuilder basicLatinWord = new System.Text.StringBuilder();
			while ((index < chars.Length) && isBasicLatin(chars[index]))
			{
				if (System.Char.IsWhiteSpace(chars[index]))
				{
					if (basicLatinWord.Length == 0)
					{
						basicLatinWord.Append(chars[index]);
						index++;
						return new Word(basicLatinWord.ToString(), org.solol.mmseg.core.IWord_Fields.BASICLATIN_WORD);
					}
					else
					{
						return new Word(basicLatinWord.ToString(), org.solol.mmseg.core.IWord_Fields.BASICLATIN_WORD);
					}
				}
				basicLatinWord.Append(chars[index]);
				index++;
			}
			
			return new Word(basicLatinWord.ToString(), org.solol.mmseg.core.IWord_Fields.BASICLATIN_WORD);
		}
		
		protected internal abstract IChunk[] createChunks(char[] chars, int index2);
		
		protected internal virtual IWord[] findMatchWords(char[] chars, int index)
		{
			IDictionary dictionary = dictionaryFactory.createDictionary();
			
			char c = chars[index];
			
			System.Collections.IList matchWords = new System.Collections.ArrayList();
			
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			sb.Append(c);
			
			System.String wordValue = sb.ToString();
			if (dictionary.isMatched(wordValue))
			{
				matchWords.Add(dictionary.getWord(wordValue));
			}
			
			for (int i = 1; i < Config.WORD_MAX_LENGTH && ((i + index) < chars.Length); i++)
			{
				if (isBasicLatin(chars[i + index]))
					break;
				sb.Append(chars[i + index]);
				System.String temp = sb.ToString();
				if (dictionary.isMatched(temp))
				{
					matchWords.Add(dictionary.getWord(temp));
				}
			}
			
			if ((matchWords.Count == 0))
			{
				matchWords.Add(new Word(wordValue, org.solol.mmseg.core.IWord_Fields.UNRECOGNIZED));
			}
			
			IWord[] words = new IWord[matchWords.Count];
			SupportClass.ICollectionSupport.ToArray(matchWords, words);
			
			matchWords.Clear();
			
			return words;
		}
		
		protected internal virtual IWord getCJKWord(IChunk[] chunks)
		{
			IChunk[] chunkList = mmRule.invoke(chunks);
			
			if (chunkList.Length >= 2)
			{
				chunkList = lawlRule.invoke(chunkList);
				if (chunkList.Length >= 2)
				{
					chunkList = svwlRule.invoke(chunkList);
					if (chunkList.Length >= 2)
					{
						chunkList = lsdmfocwRule.invoke(chunkList);
						if (chunkList.Length >= 2)
						{
							return this.ambiguityHandler.process(chunkList).Words[0];
						}
					}
				}
			}
			
			IChunk chunk = chunkList[0];
			
			return chunk.Words[0];
		}
		
		private bool isBasicLatin(char c)
		{
			return SupportClass.UnicodeSetSupport.BlockOf(c) == SupportClass.UnicodeSetSupport.BasicLatin;
		}
	}
}